## BIT Parser
## Auth: H McMILLAN
## See LICENSE file
## (c) Copyright Productivity Commission 2019 


parse_bit <- function(
  select       = NULL,
  collapse     = TRUE,
  collapse_fun = "max",
  clean_id     = TRUE,
  verbose      = TRUE,
  paths        = 
    c(
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0102.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0203.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0304.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0405.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0506.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0607.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0708.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0809.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_0910.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_1011.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_1112.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_1213.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_1314.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_1415.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_1516.csv",
      "V:/BLADE1617_CORE/CSV/IN25_BLADE1617_BIT_1617.csv"
    ),
  ...
){
  ## Start timer
  start.time <- proc.time()
  
  ## Argument check
  stopifnot(is.character(select) | is.null(select))
  stopifnot(is.character(paths))
  stopifnot(all(file.exists(paths)))
  stopifnot(collapse_fun %in% c("max", "min", "mean", "sum") | is.function(collapse_fun))
  
  ## Required package
  if(!("data.table" %in% (.packages()))) library(data.table)
  
  ## Define fread with name harmoniser & collapse
  collapse_fread <- function(path, cols = select, ...){
    
    ## Get date (seperate to parse incase not selected)
    dt_names <- names(fread(path, nrow = 0))
    date_str <- stringr::str_subset(dt_names, "BIT_Comp_")[1]
    date_str <- stringr::str_sub(date_str, -5, -1)
    
    ## Parse
    message("Parsing ", basename(path))
    dt <- fread(path, select = cols, stringsAsFactors = T, ...)
    
    ## Harmonise names and make date col
    dt[, date := date_str]
    names(dt) <- stringr::str_remove_all(names(dt), date_str)
    
    ## Collapse
    if(collapse){
      
      message("Collapsing ", basename(path))
      
      if(!is.function(collapse_fun)){
        fun_list <- list(
          min  = function(x) do.call(pmin,     c(x, na.rm = T)),
          max  = function(x) do.call(pmax,     c(x, na.rm = T)),
          mean = function(x) do.call(rowMeans, c(list(x), na.rm = T)),
          sum  = function(x) do.call(rowSums,  c(list(x), na.rm = T))
        )
        collapse_fun <- fun_list[[collapse_fun]]
      }
      
      to_collapse <- stringr::str_subset(names(dt), "^c_|^i_|^p_|^t_")
      to_collapse <- stringr::str_remove(to_collapse, "^c_|^i_|^p_|^t_")
      to_collapse <- unique(to_collapse)
      
      lapply(to_collapse, function(column){
        match_names <- stringr::str_subset(names(dt), paste0(column, "$"))
        dt[, (column) := collapse_fun(.SD), .SDcols = match_names]
        dt[, (match_names) := NULL]
      })
      
    }
    
    return(dt)
  }
  
  ## List files and read in
  bit <- lapply(paths, collapse_fread)
  
  ## Name and rowbind
  message("Binding datasets")
  bit <- rbindlist(bit, fill = T)
  
  ## Clean dates
  message("Adding new `date` column (FY end)")
  dates <- seq.Date(
    as.Date("2001-06-30"),
    as.Date("2017-06-30"),
    "years"
  )
  bit[, date := stringr::str_sub(date, -2, -1)]
  bit[, date := dates[as.integer(date)]]
  
  ## Clean IDs
  if("id" %in% names(bit) & clean_id){
    message("Reclassing `id` to integer")
    bit[, id := as.integer(stringr::str_sub(id, 3, -1))]
  }
  
  ## Stop timer
  end.time <- proc.time()
  
  ## Message duration
  duration.time <- round((end.time - start.time)[3])
  duration.time <- lubridate::seconds_to_period(duration.time)
  duration.time <- paste("Time to parse BIT:", duration.time)
  message(duration.time)
  
  ## Return bound data
  return(bit)
}