## PAYG Parser
## Auth: H McMILLAN
## See LICENSE file
## (c) Copyright Productivity Commission 2020


parse_payg <- function(
  select   = NULL,
  clean_id = TRUE,
  paths    = 
    c(
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0102.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0203.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0304.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0405.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0506.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0607.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0708.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0809.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_0910.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_1011.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_1112.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_1213.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_1314.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_1415.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_1516.csv",
      "V:/BLADE1617_CORE/CSV/IN23_BLADE1617_PAYG_1617.csv"
    ),
  ...
){
  ## Start timer
  start_time <- proc.time()
  
  ## Argument check
  stopifnot(is.character(select) | is.null(select))
  stopifnot(is.character(paths))
  stopifnot(all(file.exists(paths)))
  
  ## Required package
  if(!("data.table" %in% (.packages()))) library(data.table)
  
  ## Define fread with messaged file name
  mfread <- function(path, cols = select, ...){
    message("Parsing ", basename(path))
    fread(path, select = cols, stringsAsFactors = T, ...)
  }
  
  ## List files and read in
  payg <- lapply(paths, mfread)
  
  ## Name and rowbind
  message("Binding datasets")
  names(payg) <- basename(tools::file_path_sans_ext(paths))
  payg        <- rbindlist(payg, fill = T, idcol = "date")
  
  ## Clean dates
  message("Adding new `date` column (FY end)")
  dates <- seq.Date(
    as.Date("2001-06-30"),
    as.Date("2017-06-30"),
    "years"
  )
  payg[, date := stringr::str_sub(date, -2, -1)]
  payg[, date := dates[as.integer(date)]]
  
  ## Clean IDs
  if("id" %in% names(payg) & clean_id){
    message("Reclassing `id` to integer")
    payg[, id := as.integer(stringr::str_sub(id, 3, -1))]
  }
  
  ## Stop timer
  end_time <- proc.time()
  
  ## Message duration
  duration_time <- round((end_time - start_time)[3])
  duration_time <- lubridate::seconds_to_period(duration_time)
  duration_time <- paste("Time to parse PAYG:", duration_time)
  message(duration_time)
  
  ## Return bound data
  return(payg)
}