## BLADE utility functions
## Auth: H McMILLAN
## See LICENSE file
## (c) Copyright Productivity Commission 2019 


## Infix style error handling

"%iferror%" <- function(a, b) tryCatch({a}, error = function(e) {b})




## Split data
split_data <- function(df, splits = NULL){
  # Defaults to 80/10/10
  if(is.null(splits)) splits <- c(
    train    = 0.8,
    validate = 0.1,
    test     = 0.1
  )
  
  g <- sample(cut(
    seq(nrow(df)),
    nrow(df) * cumsum(c(0, splits)),
    labels = names(splits)
  ))
  
  return(split(df, g))
}



## Fast NA to 0 by Matt Dowle
dt_rm_na <- function(dt, na_replacemnt = 0){
  for (j in seq_len(ncol(dt))){
    if(!is.factor(dt[[j]])) set(dt, which(is.na(dt[[j]])), j, na_replacemnt)
  }
}



## Quick transfer to excel via csv

to_excel <- function(df){
  
  temp <- tempfile(fileext = ".csv", pattern = deparse(substitute(df)))
  data.table::fwrite(df, temp) %iferror%
    write.csv(df, temp, row.names = F)
  shell.exec(temp)
  
}



# Check BLADE names

# check_names <- function(select, core_dir = "V:/BLADE1617_CORE/"){
#   
#   # Load data.table
#   if(!("data.table" %in% (.packages()))) library(data.table)
#   
#   # Header extraction function
#   header <- function(dataset){
#     paths <- list.files(
#       path        = core_dir, 
#       pattern     = paste0(dataset, ".*CSV"),   
#       full.names  = T, 
#       recursive   = T, 
#       ignore.case = T
#     )
#     headers <- unlist(lapply(paths, function(p) names(fread(p, nrows = 0))))
#     data.table(name = unique(headers), data = dataset)
#   }
#   
#   # Collect headers
#   vars <- rbindlist(
#     list(
#       header("FRAME"),
#       header("BIT"),
#       header("BAS"),
#       header("PAYG")
#     )
#   )
#   
#   # Add names list without BIT type of organisation prefix
#   vars[, sans_bit_pre := gsub("^C_|^I_|^P_|^T_", "", name, ignore.case = T)]
#   
#   # Remove mandatory merging variables from select
#   select <- select[grep("^id|^date", select, ignore.case = T, invert = T)]
#   
#   # Check for select values without matches
#   no_match <- select[
#     sapply(
#       select, 
#       function(sel){
#         sel <- paste0("^", sel)
#         !any(grepl(sel, c(vars$name, vars$sans_bit_pre), ignore.case = T))
#       } 
#     )
#     ]
#   
#   # Remove values without matches
#   select <- select[!(select %in% no_match)]
#   
#   # Loop through datasets and drop non-matches in vars
#   # Used a loop to prevent simple variable names (e.g. div) from
#   # matching to longer names in datasets like BIT - hense the 
#   # dataset loop order.
#   for(dset in c("FRAME", "PAYG", "BAS", "BIT")){
#     # Find matching vars as recorded in vars dataset
#     matched_vars <- lapply(
#       select, 
#       function(sel){
#         vars[data == dset, c(name, sans_bit_pre)][
#           grepl(sel, vars[data == dset, c(name, sans_bit_pre)], ignore.case = T)
#           ]
#       } 
#     )
#     # Find matching vars as recorded in select vector
#     matched_select <- select[
#       sapply(
#         select, 
#         function(sel){
#           any(grepl(sel, vars[data == dset, c(name, sans_bit_pre)], ignore.case = T))
#         } 
#       )
#       ]
#     # Only keep uniqe matches (duplicates due to sans_bit_pre)
#     matched_vars <- unique(unlist(matched_vars))
#     # Drop non-match from vars
#     vars[data == dset & !(name %in% matched_vars), drop := T]
#     vars <- vars[is.na(drop)]
#     # Drop matched variables from select before next loop
#     select <- select[!(select %in% matched_select)]
#   }
#   
#   #Remove drop and sans_bit_pre column from vars
#   vars[, c("drop", "sans_bit_pre") := NULL]
#   
#   # Message parse summary
#   message("Matches:\n")
#   message(paste0(capture.output(vars), collapse = "\n"), "\n")
#   if(length(no_match > 0)){
#     warning(paste(no_match, collapse = ", "), " not available in BLADE core CSV files")
#   } 
# }
